#include <algorithm>
#include "include/ClosedLoopMotorController.hpp"
#include "include/Motors.hpp"
#include "include/Encoders.hpp"

#define MS_IN_S 1000
#define US_IN_S (1000 * MS_IN_S)

void ClosedLoopMotorController::set_power(uint8_t port, double power) {
  control_modes.at(port) = ControlMode::POWER;
  Motors::set_power(port, power);
}

void ClosedLoopMotorController::set_speed(uint8_t port, double speed_rpm) {
  speed_targets.at(port) = speed_rpm;
  if (control_modes.at(port) != ControlMode::SPEED) {
    pids.at(port) = PID(
        ROBOT_MOTOR_SPEED_CONTROL_KP, ROBOT_MOTOR_SPEED_CONTROL_KI, ROBOT_MOTOR_SPEED_CONTROL_KD,
        ROBOT_MOTOR_SPEED_CONTROL_RAMP, 100.0);
    control_modes.at(port) = ControlMode::SPEED;
  }
}

ClosedLoopMotorController::ClosedLoopMotorController() {
  speed_control_thread = std::thread(&ClosedLoopMotorController::speed_control_loop, this);
  speed_control_thread.detach();
}

[[noreturn]] void ClosedLoopMotorController::speed_control_loop() {
  auto sleep_duration = std::chrono::microseconds(US_IN_S / ROBOT_MOTOR_SPEED_CONTROL_RATE_HZ);
  while (true) {
    std::this_thread::sleep_for(sleep_duration);

    bool should_control_speed = std::find(control_modes.begin(), control_modes.end(), ControlMode::SPEED) != control_modes.end();
    if (!should_control_speed) {
      continue;
    }

    auto speeds = Encoders::getInstance().get_velocities_rpm();
    for (int i = 0; i < ROBOT_MOTOR_COUNT; i++) {
      if (control_modes.at(i) == ControlMode::SPEED) {
        double power = pids.at(i)(speed_targets.at(i), speeds.at(i));
        Motors::set_power(i, power);
      }
    }
  }
}


