#include "include/Encoders.hpp"
#include "include/spi.hpp"
#include "include/mathUtils.hpp"
#include <spdlog/spdlog.h>

std::array<int32_t, ROBOT_MOTOR_COUNT> Encoders::get_positions() {
  if (position_cache.is_expired()) {
    uint8_t data[ROBOT_MOTOR_COUNT * 4] = {0};
    Spi::getInstance().read_array(Spi::MOTOR_1_POS_B3, 4 * ROBOT_MOTOR_COUNT, data);
    for (int i = 0; i < ROBOT_MOTOR_COUNT; ++i) {
      cached_positions.at(i) = mathUtils::from_bytes<int32_t>(data + i * 4, 4);
    }
    position_cache.update();
  }
  return cached_positions;
}

std::array<double, ROBOT_MOTOR_COUNT> Encoders::get_velocities_rpm() {
  if (velocity_cache.is_expired()) {
    uint8_t data[ROBOT_MOTOR_COUNT * 2] = {0};
    Spi::getInstance().read_array(Spi::MOTOR_1_VEL_H, 2 * ROBOT_MOTOR_COUNT, data);
    for (int i = 0; i < ROBOT_MOTOR_COUNT; ++i) {
      auto velocity_tps = mathUtils::from_bytes<int16_t>(data + i * 2, 2);
      cached_velocities_rpm.at(i) = velocity_tps * 60.0 / ROBOT_TICKS_PER_TURN;
    }
    velocity_cache.update();
  }
  return cached_velocities_rpm;
}
